Exception: Scaltainer::ConfigurationError

Inherits:
RuntimeError
  • Object
show all
Defined in:
lib/scaltainer/exceptions.rb

Overview

Permanent misconfiguration. Retrying cannot fix it, so it is left to bubble up and terminate the process instead of being logged on every tick.